%########################
%     LINEAR MODELS
%########################

% Select flight point, trim & linearize
% -------------------------------------
%
%  StateName = u v w p q r x y z phi teta psi xth xa xe xr
%  InputName = dth da de dr wx wy wz
%  OutputName = nx ny nz p q r phi teta psi ...
%               alfa beta vc va vgr chi vz hlg dx dy dz
%
%  VC=70, XCG=0.21, MASS=150,
%  A/C is initialized 30 m below nominal trajectory and 20 m on the right
%
%
% Extract Longitudinal & Lateral Models
% -------------------------------------
%  
%        | inputs  : dth,de (1,3)
%  LONGI | outputs : Vc,Vz,q,theta,nz (11,14,5,8,3)
%        | states  : u,w,q,teta,xth,xe (1,3,5,11,13,15)
%
%        | inputs :  da,dr (2,4)
%  LAT   | outputs : ny,beta,p,r,phi (2,24,4,6,7)
%        | states : v,p,r,phi,xa,xr (2,4,6,10,14,16)
%
myroot='/home/..../';
path(path,[myroot,'CALC']); 


close all

global ACSP
clear flightpar;
flightpar.dZ=-30;
flightpar.dY=20;

MASS=[150 120 120 180 180];
XCG=[21 15 40 15 40];
VCAPP=2.572+0.5144*max(119*sqrt(MASS/140),118);

LONstateElim=[1 3 5 7 8 9 11 12 13 15];
LATstateElim=[2 4 6 7 8 9 10 12 14 16];
LONouputs=[11,14,5,8,3]; % Vc,Vz,q,theta,nz
LONinputs=[1 3];         % dth,de
LATouputs=[2,24,4,6,7];  % ny,beta,p,r,phi
LATinputs=[2 4];         % da,dr
LONOLEV=[]; LATOLEV=[];
clear sysLON sysLAT;
for ii=1:5
    flightpar.VC=VCAPP(ii);
    flightpar.MASS=MASS(ii);
    flightpar.XCG=XCG(ii);
    sys=ACStrim(flightpar);
    sysLON{ii}=modred(sys(LONouputs,LONinputs),LATstateElim,'truncate');
    sysLAT{ii}=modred(sys(LATouputs,LATinputs),LONstateElim,'truncate');
    LONOLEV=[LONOLEV eig(sysLON{ii})];
    LATOLEV=[LATOLEV eig(sysLAT{ii})];
end;

figure(1); clf; plot(LONOLEV(1:4,:),'*','MarkerSize',10);
legend('m=150t, x=20','m=120t, x=15','m=120t, x=40','m=180t, x=15',...
    'm=180t, x=40');
sgrid;
figure(2); clf; plot(LONOLEV(1:4,:),'*','MarkerSize',10);
legend('m=150t, x=20','m=120t, x=15','m=120t, x=40','m=180t, x=15',...
    'm=180t, x=40');
axis([-0.01 0 -0.2 0.2]);
sgrid;
figure(3); clf; plot(LATOLEV(1:4,:),'*','MarkerSize',10);
legend('m=150t, x=20','m=120t, x=15','m=120t, x=40','m=180t, x=15',...
    'm=180t, x=40');
sgrid;
figure(4); clf; plot(LATOLEV(1:4,:),'*','MarkerSize',10);
legend('m=150t, x=20','m=120t, x=15','m=120t, x=40','m=180t, x=15',...
    'm=180t, x=40');
axis([-0.04 0 -1 1]);
sgrid;


%################################################
%  LONGI & LAT CONTROL DESIGN VIA MODAL APPROACH
%################################################

% -----------------------------------------
% Longitudinal contoller (vc,vz)
% u  = KLON*[int(vcc-vc,vzc-vz) vc vz q az]'
%    + HLON*[vcc vzc]'
% -----------------------------------------

close all;
close_system;
sysLOND=sysLON{1};
designKLON;
[A,B,C,D]=linmod('designKLON');
Cvc=C(3,:); Cvz=C(4,:);
n=8; m=2;
In=eye(n);
Cw=zeros(1,m);


% 1/ Feedback design (KLON)
% -------------------------
%
% vc-allocated poles : 1 pair of complex poles
lvc=xxx ; % complex eigenvalue (the conjugate is automatically generated)

% vz-allocated poles : 2 pairs of complex poles
lvz1=xxx ; % complex eigenvalue (the conjugate is automatically generated)
lvz2=xxx ; % complex eigenvalue (the conjugate is automatically generated)


% gain computation
lambda=[lvc conj(lvc) lvz1 conj(lvz1) lvz2 conj(lvz2)];
Cdec=[Cvz;Cvz;Cvc;Cvc;Cvc;Cvc];
V=[]; W=[];
for ii=1:length(lambda),
  cdec=[Cdec(ii,:) Cw];
  sii=null([A-lambda(ii)*In,B;Cdec(ii,:) Cw]);
  vi=sii(1:n,:);
  wi=sii(n+1:n+m,:);
  V=[V vi];
  W=[W wi];
end;

KLONmodal=real(W/(C*V));
damp(A+B*KLONmodal*C)


% 2/ Feedforward design (HLON)
% ----------------------------
% Addition of a feed-forward term to accelerate Vz response
% u = KLON*[int(vcc-vc,vzc-vz) vc vz q az]' + HLON*[vcc vzc]'
% the term is added on the elevator command
% delta_Ec = delta_Ec + hvzc*vzc

hvzc=xxx; % tune a small (negative) coupling term
HLONmodal=[0 0;0 hvzc]; % vz-response acceleration via delta_E

% 3/ Linear Step Responses in Nominal Case
% ----------------------------------------
close_system;
sysLONS=sysLON{1};
simKLON;

KLON=KLONmodal;
HLON=zeros(2); % Without Feed-Forward
step_vcc=1; step_vzc=0;
[t,x,y]=sim('simKLON');
figure(1); clf;
subplot(211)
plot(t,y(:,1));
hold on; plot(t,y(:,2),'m'); grid;
legend('vc (m/s)','vz (m/s)','Location','NorthWest');
plot([0 max(t)],0.95*step_vcc*[1 1],'k--');
plot([0 max(t)],1.05*step_vcc*[1 1],'k--');
subplot(212)
plot(t,y(:,5)*100);
hold on; plot(t,y(:,6)*57.3,'m'); grid;
legend('\delta_{th} (%)','\delta_e (deg)','Location','NorthEast');
xlabel('Time (sec)');

step_vcc=0; step_vzc=1;
[t,x,y]=sim('simKLON');
figure(2); clf; 
subplot(211);
plot(t,y(:,1));
hold on; plot(t,y(:,2),'m'); grid;
legend('vc (m/s)','vz (m/s)','Location','SouthEast');
plot([0 max(t)],0.95*step_vzc*[1 1],'k--');
plot([0 max(t)],1.05*step_vzc*[1 1],'k--');
subplot(212);
plot(t,y(:,5)*100);
hold on; plot(t,y(:,6)*57.3,'m'); grid;
legend('\delta_{th} (%)','\delta_e (deg)','Location','SouthEast');
xlabel('Time (sec)');

% => With Feed-Forward
KLON=KLONmodal;
HLON=HLONmodal;
step_vcc=0; step_vzc=1;
[t,x,y]=sim('simKLON');
figure(2); 
subplot(211);
plot(t,y(:,2),'m--');
subplot(212);
plot(t,y(:,5)*100,'--');
hold on; plot(t,y(:,6)*57.3,'m--');


% 4/ Robustness evaluation vs Mass and Xcg 
% ----------------------------------------
KLON=KLONmodal;
HLON=HLONmodal;
LONCLEV=[];
for ii=1:5
    sysLONS=sysLON{ii};
    sysLONCi=linmod('simKLON');
    EVi=eig(sysLONCi.a); EVi=EVi(find(EVi>-5));
    LONCLEV=[LONCLEV EVi];
end;
figure(3); clf; plot(LONCLEV,'*','MarkerSize',10);
legend('m=150t, x=20','m=120t, x=15','m=120t, x=40','m=180t, x=15',...
    'm=180t, x=40','Location','SouthWest');
sgrid;


% -----------------------------------------
% Lateral contoller (Vy,phi)
% u  = KLAT*[int(nyc-ny,phic-phi) ny p r phi]'
%    + HLAT*[nyc phic]'
% -----------------------------------------


% 1/ Feedback design (KLAT)
% --------------------------
close_system;
sysLATD=sysLAT{1};
designKLAT; 

[A,B,C,D]=linmod('designKLAT');
sys=ss(A,B,C,D);
Cny=C(3,:); Cphi=C(6,:);
n=size(A,1); m=2;
In=eye(n);
Cw=zeros(1,m);

lny1=xxx;  % complex pole
lny2=xxx;  % real pole
lphi1=xxx; % complex pole
lphi2=xxx; % real pole

lambda=[lny1 conj(lny1) lny2 lphi1 conj(lphi1) lphi2];
Cdec=[Cphi;Cphi;Cphi;Cny;Cny;Cny];
V=[]; W=[];
for ii=1:length(lambda),
  cdec=[Cdec(ii,:) Cw];
  sii=null([A-lambda(ii)*In,B;Cdec(ii,:) Cw]);
  vi=sii(1:n,:);
  wi=sii(n+1:n+m,:);
  V=[V vi];
  W=[W wi];
end;
KLATmodal=real(W/(C*V));
damp(A+B*KLATmodal*C)


% 2/ Feedforward design (HLAT)
% ----------------------------
% Addition of feed-forward term to accelerate ny response and improve
% decoupling during transient
% u = KLAT*[int(nyc-ny,phic-phi) ny p r phi]' + HLAT*[nyc phic]'
% the term is added on the rudder command
% delta_Rc = delta_Er + hnyc*nyc

hnyc=xxx; % small positive coupling term to be tuned
HLATmodal=[0 0;hnyc 0];


% 3/ Linear Step Responses 
% ------------------------
close_system;
 
% without FF
sysLATS=sysLAT{1};
simKLAT;

KLAT=KLATmodal;
HLAT=zeros(2);
step_nyc=0.5;
step_phic=0;
[t,x,y]=sim('simKLAT',15);
figure(4); clf;
subplot(211)
plot(t,y(:,1));
hold on; plot(t,y(:,5)*57.3,'m'); grid;
plot(t,y(:,2)*57.3,'r'); 
legend('ny (m/s2)','phi (deg)','beta (deg)','Location','NorthEast');
subplot(212)
plot(t,y(:,6)*57.3,'m');
hold on; plot(t,y(:,7)*57.3); grid;
legend('\delta_a (deg)','\delta_r (deg)','Location','SouthEast');
xlabel('Time (sec)');

step_nyc=0;
step_phic=10/57.3;
[t,x,y]=sim('simKLAT',15);
figure(5); clf; 
subplot(211);
plot(t,y(:,1));
hold on; plot(t,y(:,5)*57.3,'m'); grid;
legend('ny (m/s2)','phi (deg)','Location','NorthEast');
subplot(212);
plot(t,y(:,6)*57.3,'m');
hold on; plot(t,y(:,7)*57.3); grid;
legend('\delta_a (deg)','\delta_r (deg)','Location','NorthWest');
xlabel('Time (sec)');

% with FF
figure(6); clf;
KLAT=KLATmodal;
HLAT=zeros(2);
step_nyc=0.5;
step_phic=0;
[t,x,y]=sim('simKLAT',15);
plot(t,y(:,1));
hold on; plot(t,y(:,5)*57.3,'m'); grid;
HLAT=HLATmodal;
[t,x,y]=sim('simKLAT',15);
plot(t,y(:,1),'--');
hold on; plot(t,y(:,5)*57.3,'m--'); 
legend('ny response without feedforward',...
    'phi response without feedforward',...
    'ny response with feedforward',...
    'phi response with feedforward','Location','NorthEast');
xlabel('Time (sec)');


% 4/ Robustness evaluation vs Mass and Xcg 
% ----------------------------------------
HLAT=HLATmodal;
KLAT=KLATmodal;
LATCLEV=[];
for ii=1:5
    sysLATS=sysLAT{ii};
    sysLATCi=linmod('simKLAT');
    EVi=eig(sysLATCi.a); EVi=EVi(find(EVi>-10));
    LATCLEV=[LATCLEV EVi];
end;
figure(7); clf; plot(LATCLEV,'*','MarkerSize',10);
legend('m=150t, x=20','m=120t, x=15','m=120t, x=40','m=180t, x=15',...
    'm=180t, x=40','Location','SouthWest');
sgrid;


%#############################################
%  IMPLEMENTATION IN NONLINEAR MODEL : ALS_PIL
%#############################################
close all;
close_system;
ALS_PIL;

% New field required : ACSP.ALS
% -----------------------------
clear ALSpar;
ALSpar.KLON=KLONmodal;
ALSpar.HLON=HLONmodal;
ALSpar.KLAT=KLATmodal;
ALSpar.HLAT=HLATmodal;
ACSP.ALS=ALSpar;

% WindSteps, flight conditions and trimming
% -----------------------------------------
WindStepTime=[10 20 30]; 
WindStepMag=[0 0 0]; 

clear flightpar
flightpar.VC=VCAPP(1);   % 65.93
flightpar.MASS=MASS(1);  % 150
flightpar.XCG=XCG(1);    % 21
flightpar.dZ=-30;
flightpar.dY=20;
ACStrim(flightpar);

% Check implementation : closed-loop poles comparison
% ---------------------------------------------------

% decoupled linear closed-loop models
step_vcc=0; step_vzc=0; step_nyc=0; step_phic=0;
sysLONS=sysLON{1};  sysLATS=sysLAT{1}; 
[ALON,BLON,CLON,DLON]=linmodv5('simKLON'); damp(ALON)
[ALAT,BLAT,CLAT,DLAT]=linmodv5('simKLAT'); damp(ALAT)

% linearization of the full nonlinear closed-loop
[t0,x0,y0]=sim('ALS_PIL',0);
[AFULL,BFULL,CFULL,DFULL]=linmodv5('ALS_PIL',x0);
damp(AFULL)

% Nonlinear simulations with step inputs
% --------------------------------------
step_vcc=3; step_vzc=0; step_nyc=0; step_phic=0;
[t,x,y]=sim('ALS_PIL',40);
figure(1); clf; 
subplot(221); plot(t,y(:,11)); grid;      % VC
ylabel('Vc'); xlabel('Time (sec)');
subplot(222); plot(t,y(:,14)); grid;       % VZ
ylabel('Vz'); xlabel('Time (sec)');
subplot(223); plot(t,y(:,26)*100); grid;  % TH
ylabel('epr (%)'); xlabel('Time (sec)');
subplot(224); plot(t,y(:,30)*57.3); grid;  % DE
ylabel('\delta_e (deg)'); xlabel('Time (sec)');

step_vcc=0; step_vzc=3; step_nyc=0; step_phic=0;
[t,x,y]=sim('ALS_PIL',20);
figure(2); clf; 
subplot(221); plot(t,y(:,11)); grid;      % VC
ylabel('Vc'); xlabel('Time (sec)');
subplot(222); plot(t,y(:,14)); grid;       % VZ
ylabel('Vz'); xlabel('Time (sec)');
subplot(223); plot(t,y(:,26)*100); grid;  % TH
ylabel('epr (%)'); xlabel('Time (sec)');
subplot(224); plot(t,y(:,30)*57.3); grid;  % DE
ylabel('\delta_e (deg)'); xlabel('Time (sec)');

step_vcc=0; step_vzc=0; step_nyc=0.5; step_phic=0;
[t,x,y]=sim('ALS_PIL',20);
figure(3); clf; 
subplot(221); plot(t,y(:,2)); grid;      % Ny
ylabel('Ny (m/s2)'); xlabel('Time (sec)');
subplot(222); plot(t,y(:,7)*57.3); grid;  % PHI
ylabel('\phi (deg)'); xlabel('Time (sec)');
subplot(223); plot(t,y(:,28)*57.3); grid;  % DA
ylabel('\delta_a (deg)'); xlabel('Time (sec)');
subplot(224); plot(t,y(:,31)*57.3); grid;  % DR
ylabel('\delta_r (deg)'); xlabel('Time (sec)');

step_vcc=0; step_vzc=0; step_nyc=0; step_phic=10/57.3;
[t,x,y]=sim('ALS_PIL',20);
figure(4); clf; 
subplot(221); plot(t,y(:,2)); grid;      % Ny
ylabel('Ny (m/s2)'); xlabel('Time (sec)');
subplot(222); plot(t,y(:,7)*57.3); grid;  % PHI
ylabel('\phi (deg)'); xlabel('Time (sec)');
subplot(223); plot(t,y(:,28)*57.3); grid;  % DA
ylabel('\delta_a (deg)'); xlabel('Time (sec)');
subplot(224); plot(t,y(:,31)*57.3); grid;  % DR
ylabel('\delta_r (deg)'); xlabel('Time (sec)');


%##############################################
%  LONGI & LAT GUIDANCE LOOPS DURING APPROACH
%##############################################

step_vcc=0; step_vzc=0; step_nyc=0; step_phic=0;
[t0,x0,y0]=sim('ALS_PIL',0);
[A,B,C,D]=linmodv5('ALS_PIL',x0);
sysPIL=ss(A,B,C,D);

% ------------------------
% GLIDE Controller design
% ------------------------
% generate transfer from vzc to DZ : GLIDE
% the guidance loop is then tuned via sisotool
% a proportional feedback gain works well
% vzc = kGLD*DZ with kGLD=0.1
sysGLD=minreal(sysPIL(19,2));
damp(sysGLD)
tzero(sysGLD)
sisotool(sysGLD)

kpGLD=xxx;  % to be tuned
kiGLD=xxx;  % to be tuned

damp(feedback(sysGLD,kpGLD))
KGLD=tf([kpGLD kiGLD],[1 0]);
damp(feedback(sysGLD,KGLD))


% ----------------------
% LOC Controller design
% ----------------------
% generate transfer from phic to DY : LOC
% a PD control is needed to compensate the double integrator
% phic = (kpLOC+kdLOC*s)*DY is first tuned with kp=0.003, kd=9*kp
% next a fast pole is added to ensure realizability = 1/(1+0.05s)
% Implementation in NL model is performed via an estimation of DYdot
% DYdot = Chi*Vground  ==>  phic = kpLOC*DY + kdLOC*Chi*Vground
sysLOC=minreal(sysPIL(18,4));
damp(sysLOC)
sisotool(sysLOC)

kpLOC=xxx;       % to be tuned
kdLOC=xxx;       % to be tuned
KLOC=tf([kdLOC kpLOC],[0.05 1]);
damp(feedback(sysLOC,KLOC))

%#########################################################
%  IMPLEMENTATION IN NONLINEAR MODEL : ALS_APP1 & ALS_APP2
%#########################################################
close all;
close_system;
ALS_APP1;

% ----------------------------------------------------------
% Update ACSP.ALS field with both control and guidance gains
% ----------------------------------------------------------
clear ALSpar;
ALSpar.KLON=KLONmodal;
ALSpar.HLON=HLONmodal;
ALSpar.KLAT=KLATmodal;
ALSpar.HLAT=HLATmodal;
ALSpar.kpGLD=kpGLD;
ALSpar.kiGLD=kiGLD;
ALSpar.kpLOC=kpLOC;
ALSpar.kdLOC=kdLOC;


% ---------------
% RUN SIMULATIONS
% ---------------
ACSP.ALS=ALSpar;

% SIMULATION #1  => Nominal Case without wind
% -------------------------------------------
clear flightpar
flightpar.VC=VCAPP(1);   
flightpar.MASS=MASS(1);  
flightpar.XCG=XCG(1);    
flightpar.dZ=-30;
flightpar.dY=20;
ACStrim(flightpar);

WindStepTime=[30 50 70];
WindStepMag=[0 0 0];
[t,~,z]=sim('ALS_APP1');
ALplots(t,z,1);

% SIMULATION #2  => Nominal Case with windsteps
% ---------------------------------------------
WindStepTime=[30 50 70];
WindStepMag=[-5 5 1];
[t,~,z]=sim('ALS_APP1');
ALplots(t,z,5);


% SIMULATION #3 : Wind and Turbulences
% ------------------------------------
clear flightpar
flightpar.MASS=150;
flightpar.XCG=21;
flightpar.dZ=-30;
flightpar.dY=20;
Wx33=-25*0.5144; % medium level
Wy33=15*0.5144;  % medium level
flightpar.WX=1.6*Wx33;
Vref=0.5144*max(119*sqrt(flightpar.MASS/140),118);
if Wx33<0
    Vapp=Vref+max(2.572,-Wx33/3);
else
    Vapp=Vref+2.572;
end;
flightpar.VC=Vapp;
ACStrim(flightpar);

ACSP.TURBW.Wx33=Wx33;
ACSP.TURBW.Wy33=Wy33;
ACSP.TURBW.sigu=0.15*sqrt(Wx33^2+Wy33^2);
ACSP.TURBW.sigw=0.5;
ACSP.TURBW.seedwx=1;
ACSP.TURBW.seedwy=2;
ACSP.TURBW.seedwz=3;

ACSP.NOISE.devLOC=0;
ACSP.NOISE.sigLOC=2.5;
ACSP.NOISE.sigGLD=10;
ACSP.NOISE.seedLOC=4;
ACSP.NOISE.seedGLD=5;

[t,~,z]=sim('ALS_APP2');
ALplots(t,z,9);
